/* PROGRAM: Parallel Heat
 * AUTHORS:
 *     Aaron Weeden (Shodor Education Foundation)
 *     Skylar Thompson (University of Washington)
 * DATE: December 2013
 *
 * See the README for instructions on compiling and running this program.
 */

#include <stdio.h>
#include <errno.h>
#include <getopt.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "heat.h"

int main(int argc, char **argv) {

    /* Data Structure (what the model has) */
    point_t *points;
    userArgs_t userArgs;
    unsigned int timeStepIndex;

    /* All ranks will fetch user arguments from get_user_args */
    if((get_user_args(argc,argv,&userArgs)) == -1) {
        fprintf(stderr,"%s: get_user_args failed\n",argv[0]);
        exit(EXIT_FAILURE);
    }

    /* Algorithm (how the model computes) */

    /* 1-10: All ranks will initialize the field of points */
    if((points = init_points(userArgs)) == NULL) {
        fprintf(stderr,"%s: Could not initialize points\n",argv[0]);
        exit(EXIT_FAILURE);
    }

    /* 
     * Start the clock - print the grid, advance the heat in the future grid, 
     * then copy the future grid back to the present grid
     */
    for (
            /* 11: start at time step 0 */
            timeStepIndex = 0;

            /* 12: are we finished? */
            timeStepIndex < userArgs.timeStepCount;

            /* 32: move on to the next time step */
            timeStepIndex++
        ) {

        /* 13-18: Print present temperature grid to the terminal */
        print_points(userArgs,points);

        /* 
         * 19-27: Set temperatures in the future grid based on temperatures in
         * the present grid
         */
        set_future_points(userArgs,points);

        /* 28-31: Copy the future grid to the present, so we can start again */
        copy_future_to_present(userArgs,points);
    }
    /* deallocate memory */
    free(points);

    exit(EXIT_SUCCESS);
}
